/*
 * JBoss, Home of Professional Open Source
 * Copyright 2011 Red Hat Inc. and/or its affiliates and other
 * contributors as indicated by the @author tags. All rights reserved.
 * See the copyright.txt in the distribution for a full listing of
 * individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.infinispan.quickstart.embeddedcache;

import java.awt.geom.Point2D;
import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.infinispan.Cache;
import org.infinispan.configuration.cache.ConfigurationBuilder;
import org.infinispan.configuration.cache.ConfigurationChildBuilder;
import org.infinispan.eviction.EvictionStrategy;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.manager.EmbeddedCacheManager;
import org.infinispan.persistence.sifs.configuration.SoftIndexFileStoreConfigurationBuilder;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

public class SoftIndexCacheStoreUpdateTest {

    public EmbeddedCacheManager manager = new DefaultCacheManager();

    private ConfigurationChildBuilder createSoftIndexConfigBuilder(String location) {
        return new ConfigurationBuilder()
                .persistence().addStore(SoftIndexFileStoreConfigurationBuilder.class)
                .indexLocation(location + "/index").dataLocation(location + "/data").maxNodeSize(1000)
                .preload(true).purgeOnStartup(false)
                .eviction().strategy(EvictionStrategy.LRU).maxEntries((long) 100);
    }

    /**
     * This test puts a value three times (can also be different values, same effect),
     * then restarts the cache, updates the value, restarts the cache again and reads the value.
     *
     * The test fails because if there are multiple puts on the key before the first restart,
     * the update is not persisted and the old value is read.
     *
     * @throws IOException
     */
    @Test
    public void testUpdate() throws IOException {
        String key = "my_key";
        String cacheName = "update-test-cache";
        File cacheDir = new File(cacheName);
        if (cacheDir.exists()) {
            FileUtils.deleteDirectory(cacheDir);
        }

        // initialization of cache
        manager.defineConfiguration(cacheName, createSoftIndexConfigBuilder(cacheName).build());
        Cache<String,Point2D.Double> cache = manager.getCache(cacheName);

        Point2D.Double originalPoint = new Point2D.Double(24, 46);
        cache.put(key, originalPoint);
        // put value two more times (this is what causes the problem)
        cache.put(key, originalPoint);
        cache.put(key, originalPoint);
        cache.stop();
        manager.stop();

        // restart cache
        manager = new DefaultCacheManager();
        manager.defineConfiguration(cacheName, createSoftIndexConfigBuilder(cacheName).build());
        Cache<String,Point2D.Double> cacheAfterFirstRestart = manager.getCache(cacheName);

        // update cache entry
        Point2D.Double pointAfterFirstRestart = cacheAfterFirstRestart.get(key);
        assertNotNull(pointAfterFirstRestart);
        assertEquals(24, pointAfterFirstRestart.getX(), 0.0);
        pointAfterFirstRestart.setLocation(33, 44);
        cacheAfterFirstRestart.put(key, pointAfterFirstRestart);
        cacheAfterFirstRestart.stop();
        manager.stop();

        // restart cache
        manager = new DefaultCacheManager();
        manager.defineConfiguration(cacheName, createSoftIndexConfigBuilder(cacheName).build());
        Cache<String,Point2D.Double> cacheAfterSecondRestart = manager.getCache(cacheName);

        // cache should contain updated entry
        Point2D.Double pointAfterSecondRestart = cacheAfterSecondRestart.get(key);
        assertNotNull(pointAfterSecondRestart);
        assertEquals("Cache should contain updated entry (33.0), not original value (24.0)",
                     33, pointAfterSecondRestart.getX(), 0.0);

        cacheAfterSecondRestart.stop();
        manager.stop();
    }


}
